package io.malloc.ccc.nc;

import java.util.Arrays;
import com.wolfssl.wolfcrypt.Aes;
import com.wolfssl.wolfcrypt.WolfCryptException;


public class AesTest {

  private static final byte[] KEY = Util.h2b("00112233445566778899AABBCCDDEEFF");
  private static final byte[] IV = Util.h2b("000102030405060708090A0B0C0D0E0F");


  public void testEncDec() {

    byte[] key = Util.h2b("2b7e151628aed2a6abf7158809cf4f3c");
    byte[] iv = Util.h2b("000102030405060708090A0B0C0D0E0F");
    byte[] in = Util.h2b("6bc1bee22e409f96e93d7e117393172a");
    byte[] expected = Util.h2b("7649abac8119b246cee98e9b12e9197d");

    byte[] cipher = null;
    byte[] plain = null;

    Aes enc = new Aes(key, iv, Aes.ENCRYPT_MODE);
    cipher = enc.update(in, 0, in.length);
    if ( Arrays.equals(expected, cipher) ) {
      System.out.println("J3G_YAY: encrypt function working!!");
      System.out.println("J3G_YAY: "+Util.b2h(expected)+"=="+Util.b2h(cipher));
    }
    else {
      System.out.println("J3G_ERR: encrypt isn't working properly");
      System.out.println("J3G_YAY: "+Util.b2h(expected)+"!="+Util.b2h(cipher));
    }

    Aes dec = new Aes(key, iv, Aes.DECRYPT_MODE);
    plain = dec.update(cipher, 0, cipher.length);
    if ( Arrays.equals(in, plain) ) {
      System.out.println("J3G_YAY: decrypt function working!!");
      System.out.println("J3G_YAY: "+Util.b2h(in)+"=="+Util.b2h(plain));
    }
    else {
      System.out.println("J3G_ERR: decrypt error");
      System.out.println("J3G_ERR: "+Util.b2h(in)+"!="+Util.b2h(plain));
    }

    /* free objects */
    enc.releaseNativeStruct();
    dec.releaseNativeStruct();

    /* try to re-init and re-use them */
    enc = new Aes(key, iv, Aes.ENCRYPT_MODE);
    cipher = enc.update(in, 0, in.length);
    if ( Arrays.equals(expected, cipher) ) {
      System.out.println("J3G_YAY: encrypt function working!!");
    }
    else {
      System.out.println("J3G_ERR: encrypt isn't working properly");
    }

    dec = new Aes(key, iv, Aes.DECRYPT_MODE);
    plain = dec.update(cipher, 0, cipher.length);
    if ( Arrays.equals(in, plain) ) {
      System.out.println("J3G_YAY: decrypt function working!!");
    }
    else {
      System.out.println("J3G_ERR: decrypt isn't working properly");
    }

    /* free again */
    enc.releaseNativeStruct();
    dec.releaseNativeStruct();
  }


  public void checkSetKeyParams() {
    /* iv is optional, should not raise. */
    Aes aes = new Aes(KEY, null, Aes.ENCRYPT_MODE);

    try {
      aes.setKey(null, IV, Aes.ENCRYPT_MODE);
    } catch (WolfCryptException e) {
      System.out.println("ERROR: WolfCrypt: J3G: "+e.getMessage());
    }

    aes.setKey(KEY, IV, Aes.ENCRYPT_MODE);
    aes.releaseNativeStruct();

    try {
      aes.setKey(KEY, IV, Aes.ENCRYPT_MODE);
    } catch (WolfCryptException e) {
      System.out.println("ERROR: WolfCrypt: J3G: "+e.getMessage());
    }
  }

}
